/*
 * This file is part of nads
 * Copyright (c) 2003 ECSC Ltd.
 * Author: Gianni Tedesco <gianni@scaramanga.co.uk>
 * Released under the terms of the GNU GPL version 2
 *
 * Because different webservers require different normalization due to
 * non RFC-strict implementations and abiguities in the standards. This
 * file keeps track of which one the user has selected.
 *
 * TODO:
 *  o What if the application is protecting many webservers.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <nads.h>

struct webserver {
	const char *name;
	unsigned long flags;
};

int __hidden nads_errcode;

/* Webserver database */
/* TODO: Needs more research and more entries */
static struct webserver webservers[]={
	{"IIS",		NADS_WS_MS_UTF16 | NADS_WS_PERCENT_HACK |
			NADS_WS_DOUBLE_HEX },
	{"Apache",	NADS_WS_CASE_SENSITIVE},
};

/* Currently selected webserver */
static struct webserver *webserver=NULL;

/* nads_select_webserver:
 * @name: Name of the webserver to protect
 *
 * Selects a webserver type to protect.
 *
 * Return value: non-zero for success, zero for error.
 */
int nads_select_webserver(const char *name)
{
	int num=sizeof(webservers)/sizeof(*webservers);
	int i;

	for(i=0; i<num; i++) {
		if ( !strcasecmp(webservers[i].name, name) ) {
			webserver = &webservers[i];
			nads_errcode = NADS_ERR_SUCCESS;
			return NADS_OK;
		}
	}

	nads_errcode = NADS_ERR_NO_WEBSERVER;
	return NADS_FAIL;
}

/* nads_ws:
 * @x: the feature(s) we are checking for
 *
 * Returns true if ALL the requested features bits are set
 * by the currently selected webserver.
 *
 * Return value: 1 for true, 0 for false
 */
int __hidden nads_ws(unsigned long x)
{
	return ( (webserver->flags & x) != 0 );
}
